@@ -1,6 +1,8 @@
 
-/* the global test counter, now living in C-land, and other symbols. */
+/* the global test counter, now living in C-land. */
 extern double test_counter;
+
+/* symbols used to set object attributes. */
 extern SEXP BN_ModelstringSymbol;
 extern SEXP BN_NodesSymbol;
 extern SEXP BN_ProbSymbol;
@@ -8,6 +10,10 @@ extern SEXP BN_MethodSymbol;
 extern SEXP BN_WeightsSymbol;
 extern SEXP BN_DsepsetSymbol;
 extern SEXP BN_MetaDataSymbol;
+extern SEXP BN_NobsSymbol;
+extern SEXP BN_DfSymbol;
+
+/* shortcuts to boolean SEXPs. */
 extern SEXP TRUESEXP, FALSESEXP;
 
 /* numerical constants */
